pub enum ArbitraryDisambiguateSeparation {
None,
Comma,
Space,
Both,
}Expand description
When defining a [PluginArbitraryMatcher] for an Arbitrary kind, defines how values
are separated.
A lot of CSS properties allow specifying several values of a single type separated by a
character, e.g margin allows <length>
or <percentage>
values separated by spaces, to define a specific margin for each side of the CSS layout box.
This enumeration helps matching these values when using a [PluginArbitraryMatcher], e.g for
the margin example, you would use
ⓘ
Plugin::new(...)
.matchers(&[Length, Percentage], PluginArbitraryMatcherSeparation::Space)Variants
None
No separation, a single value is matched.
Comma
Separated by commas (,).
Example: 33px, 42%, 6em.
Space
Separated by spaces ( ).
Example: left top.
Both
Separated by commas (,) then spaces ( ).
Example: left, 12% 33px, right center.