StaticPlugin

Type Alias StaticPlugin 

pub type StaticPlugin = Plugin<&'static str, &'static [&'static str], Map<&'static str, &'static str>, Map<&'static str, &'static [&'static str]>, &'static [CssType]>;
An alias to a Plugin which can easily be defined in Rust, e.g in const environments.

It requires using &'static str for all configuration. If you need to use Strings for some dynamic configuration, use DynamicPlugin instead.

Aliased Type

pub enum StaticPlugin {
    ListProperties(ListProperties<&'static str, &'static [&'static str], Map<&'static str, &'static str>, Map<&'static str, &'static [&'static str]>>),
    ListValues(ListValues<&'static str, &'static [&'static str], Map<&'static str, &'static str>>),
    Spacing(Spacing<&'static str, &'static [&'static str], Map<&'static str, &'static str>>),
    Color(Color<&'static str, &'static [&'static str], Map<&'static str, &'static str>>),
    Number(Number<&'static str, &'static [&'static str], Map<&'static str, &'static str>>),
    Arbitrary(Arbitrary<&'static str, &'static [&'static str], Map<&'static str, &'static str>, &'static [CssType]>),
    Functional(Functional<&'static str>),
}

Variants



ListProperties(ListProperties<&'static str, &'static [&'static str], Map<&'static str, &'static str>, Map<&'static str, &'static [&'static str]>>)



ListValues(ListValues<&'static str, &'static [&'static str], Map<&'static str, &'static str>>)



Spacing(Spacing<&'static str, &'static [&'static str], Map<&'static str, &'static str>>)

See Spacing.



Color(Color<&'static str, &'static [&'static str], Map<&'static str, &'static str>>)

See Color.



Number(Number<&'static str, &'static [&'static str], Map<&'static str, &'static str>>)

See Number.



Arbitrary(Arbitrary<&'static str, &'static [&'static str], Map<&'static str, &'static str>, &'static [CssType]>)

See Arbitrary.



Functional(Functional<&'static str>)

See Functional.

Not serializable.