Enum Modifier Copy item path Source pub enum Modifier<'a> {
Builtin {
is_negative: bool ,
value: &'a str ,
},
Arbitrary {
hint: Option <CssType >,
value: Cow <'a, str >,
},
}Expand description The modifier is the rest of the selector after the namespace, it is used to clarify the
CSS needed to be generated.
A builtin static modifier (e.g. bg-red-500).
Fields Whether the value is negative (e.g. -translate-2 is negative).
The inner value of the modifier.
A dynamic modifier capable of automatically generating a rule from a CSS value
(e.g. bg-[rgb(12_12_12)]).
All underscores in the value will be replaced by spaces except in url(), if you really
want to keep one of them, you can prefix it with a backslash \_ and it will be used as
is.
Sometimes the value is ambiguous, for example bg-[var(--foo)] can be handled by either
the background color or the
background size utility. In this case,
you need to provide a CSS type
hint before the arbitrary value. For example
bg-[length:var(--foo)] will generate background-size: var(--foo); (using the
background size utility).
See CssType for a list of type hints.
Fields The type hint needed for ambiguous values.
The inner value of the modifier.
All escaped characters (prefixed by a backslash) are already unescaped.
Performs copy-assignment from
source.
Read more Formats the value using the given formatter.
Read more Compares and returns the maximum of two values.
Read more Compares and returns the minimum of two values.
Read more Restrict a value to a certain interval.
Read more 🔬 This is a nightly-only experimental API. (clamp_to)
Restrict a value to a certain range.
Read more This method returns an ordering between
self and
other values if one exists.
Read more Tests less than (for
self and
other) and is used by the
< operator.
Read more Tests less than or equal to (for
self and
other) and is used by the
<= operator.
Read more Tests greater than (for
self and
other) and is used by the
>
operator.
Read more Tests greater than or equal to (for
self and
other) and is used by
the
>= operator.
Read more Immutably borrows from an owned value.
Read more Mutably borrows from an owned value.
Read more 🔬 This is a nightly-only experimental API. (clone_to_uninit)
Performs copy-assignment from
self to
dest.
Read more Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From <T> for U chooses to do.
The resulting type after obtaining ownership.
Creates owned data from borrowed data, usually by cloning.
Read more Uses borrowed data to replace owned data, usually by cloning.
Read more The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.