Enum VariantType

pub enum VariantType {
    PseudoElement(&'static str),
    PseudoClass(&'static str),
    WrapClass(Cow<'static, str>),
    AtRule(Cow<'static, str>),
    Group(&'static str),
    Peer(&'static str),
    PeerNot(&'static str),
}
Structure used to add pseudo-selectors, pseudo-elements, pseudo classes and media queries to CSS rules.

Variant are useful to conditionally apply utility classes.

See config::BUILTIN_VARIANTS for a list of all default variants.

See Tailwind’s documentation to learn more about variants.

Variants



PseudoElement(&'static str)

A CSS pseudo element.

Example

If the variant is VariantType::PseudoClass("before") and the original class is ".bg-red-500", the class will become ".bg-red-500::before").



PseudoClass(&'static str)

A CSS pseudo class.

Example

If the variant is VariantType::PseudoClass("hover") and the original class is ".bg-red-500", the class will become ".bg-red-500:hover").



WrapClass(Cow<'static, str>)

Wrap the original class to make another one.

Example

If the variant is VariantType::WrapClass("&[open]") and the original class is ".bg-red-500", the class will become ".bg-red-500[open]").



AtRule(Cow<'static, str>)

Add a @ CSS rule (like @media, @supports).

Example

If the variant is VariantType::AtRule("@media (orientation: portrait)") and the original class is ".bg-red-500", the class will become "@media (orientation: portrait) { .bg-red-500 { ... } }").



Group(&'static str)

A variant applied to a group element like group-hover.

This variant should not be built manually.



Peer(&'static str)

A variant applied to a peer element like peer-focus.

This variant should not be built manually.



PeerNot(&'static str)

A negated variant applied to a peer element like peer-not-hover.

This variant should not be built manually.